function [ stat ] = mrLinearStat1d( u, interval, weights, scalePenalties )
%MRLINEARSTAT1D Multiresolution linear statistics in 1-dimension  
%It computes
%
%   max (w_I)^(-1) | < e_I, u > | - q_I,
% 
%where w_I weights
%      e_I indicator of interval I
%      q_I scale penalties
%
%Usage
%
%   mr = mrLinearStat1d(u, cube, weights, scalePenalties)
%   mr = mrLinearStat1d(u, cube, weights)
%   mr = mrLinearStat1d(u, cube)
%
%Input
%   u               a matrix
%   interval        a list (li, ri) li and ri are nInterval x 1 vectors
%   weights         a vector specifying weights (by default sqrt(#samples))
%   scalePenalties  a vector specifying scale penalties (by defaul 0)

% Housen Li @ MPIbpc
% 01.11.2015 create from 'mrLinearStat2d.m' 

if (nargin < 3) || isempty(weights) 
    weights = sqrt(interval.ri-interval.li+1);
end
if (nargin < 4) || isempty(scalePenalties)
    scalePenalties = zeros(length(weights),1);
end

% C implementation
stat = mrLinearStat1d_mex(u(:), interval.li(:)-1, interval.ri(:)-1, weights(:), scalePenalties(:));

