// H. Li - June 3, 2015
// This is a stand-alone file.

//  mex -Dchar16_t=UINT16_T mrstat_mex.c
#include "mex.h"
#include <math.h>

void mrcoefIntval(double *, const double*, int, const double*, const double*, int);

// coef = mrcoefIntval_mex(y, li, ri)
// it works only for 1d
// y:  data, column vector
// li: left end indices of input intervals, start from 0, column vector
// ri: right end indices of input intervals, start from 0, column vector
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double *y  = mxGetPr(prhs[0]);
    int n      = mxGetM(prhs[0]);
    double *li = mxGetPr(prhs[1]);
    double *ri = mxGetPr(prhs[2]);
    int nIntv  = mxGetM(prhs[1]);
        
    plhs[0] = mxCreateDoubleMatrix(nIntv, 1, mxREAL);
    double *coef = mxGetPr(plhs[0]);
    
    mrcoefIntval(coef, y, n, li, ri, nIntv);
    
    return;
}

// multiscale statistics
void mrcoefIntval(double *coef,                          /* multiscale coefficients (output) */
                    const double *y, int n,                /* whole data */
            const double *li, const double *ri, int nIntv) /* specifying system of cubes */
{
    int i;
    double lcs;
    // cumulative sum
    double *cs = calloc(n+1,sizeof(double));
    cs[0] = 0.;
    for (i = 0; i < n; ++i) {
        cs[i+1] = y[i] + cs[i];
    }
    
    for (i = 0; i < nIntv; ++i) {
        coef[i] = (cs[(int)ri[i]+1] - cs[(int)li[i]]) / sqrt(ri[i]+1-li[i]);
    }
    free(cs);
    
    return;
}