// H. Li - June 3, 2015
// This is a stand-alone file.

//  mex -Dchar16_t=UINT16_T mrstat_mex.c
#include "mex.h"
#include <math.h>

void mrdualIntval(double*, int, const double*, const double*, const double*, int);

// u = mrdualIntval_mex(n, coef, li, ri)
// it works only for 1d
// n:    size of original space
// coef: coefficients
// li:   left end indices of input intervals, start from 0, column vector
// ri:   right end indices of input intervals, start from 0, column vector
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    int n        = (int)mxGetScalar(prhs[0]);
    double *coef = mxGetPr(prhs[1]);
    double *li   = mxGetPr(prhs[2]);
    double *ri   = mxGetPr(prhs[3]);
    int nIntv    = mxGetM(prhs[2]);
        
    plhs[0] = mxCreateDoubleMatrix(n, 1, mxREAL);
    double *u = mxGetPr(plhs[0]);
    
    mrdualIntval(u, n, coef, li, ri, nIntv);
    
    return;
}

// multiscale statistics
void mrdualIntval(double *u, int n,                        /* original space (output) */
                    const double *coef,                    /* multiscale coefficients */
            const double *li, const double *ri, int nIntv) /* specifying system of cubes */
{
    int i, j;
    double sc;
    for (i = 0; i < n; ++i) {
        u[i] = 0.;
    }
    for (i = 0; i < nIntv; ++i) {
        sc = coef[i] / sqrt(ri[i] - li[i] + 1);
        for (j = (int)li[i]; j <= (int)ri[i]; ++j) {
            u[j] += sc;
        }
    }
    return;
}