// H. Li - June 2, 2015
// This is a stand-alone file.

//  mex -Dchar16_t=UINT16_T mrstat_mex.c
#include "mex.h"
#include <math.h>

double mrstatIntval(const double*, int, const double*, const double*, int);

// s = mrstatIntval_mex(y, li, ri)
// it works only for 1d
// y:  data, column vector
// li: left end indices of input intervals, start from 0, column vector
// ri: right end indices of input intervals, start from 0, column vector
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    double *y  = mxGetPr(prhs[0]);
    int n      = mxGetM(prhs[0]);
    double *li = mxGetPr(prhs[1]);
    double *ri = mxGetPr(prhs[2]);
    int nIntv  = mxGetM(prhs[1]);
    
    double ret = mrstatIntval(y, n, li, ri, nIntv);
    
    // return value
    plhs[0] = mxCreateDoubleScalar(ret);
}

// multiscale statistics
double mrstatIntval(const double *y, int n,                /* whole data */
            const double *li, const double *ri, int nIntv) /* specifying system of cubes */
{
    int i;
    double ret, lcs;
    // cumulative sum
    double *cs = calloc(n+1,sizeof(double));
    cs[0] = 0.;
    for (i = 0; i < n; ++i) {
        cs[i+1] = y[i] + cs[i];
    }
    
    ret = -INFINITY;
    // cumulative sum in column-direction
    for (i = 0; i < nIntv; ++i) {
        lcs = fabs(cs[(int)ri[i]+1] - cs[(int)li[i]]) / sqrt(ri[i]+1-li[i]);
        ret = fmax(ret, lcs);
    }
    free(cs);
    
    return ret;
}