function [ stat ] = mrLinearStat2d( u, cube, weights, scalePenalties )
%MRLINEARSTAT2D Multiresolution linear statistics in 2-dimension  
%It computes
%
%   max (w_B)^(-1) | < e_B, u > | - q_B,
% 
%where w_B weights
%      e_B indicator of cube B
%      q_B scale penalties
%
%Usage
%
%   mr = mrLinearStat2d(u, cube, weights, scalePenalties)
%   mr = mrLinearStat2d(u, cube, weights)
%   mr = mrLinearStat2d(u, cube)
%
%Input
%   u               a matrix
%   cube            a list (st, ed) st and ed are nCube x 2 matrices
%   weights         a vector specifying weights (by default sqrt(#samples))
%   scalePenalties  a vector specifying scale penalties (by defaul 0)
%Output
%   stat    the multiresolution linear statistics

% Housen Li @ MPIbpc
% 30.09.2015 create the code 

if (nargin < 3) || isempty(weights) 
    weights = sqrt(prod(cube.ed-cube.st+1,2));
end
if (nargin < 4) || isempty(scalePenalties)
    scalePenalties = zeros(length(weights),1);
elseif isscalar(scalePenalties)
    scalePenalties = scalePenalties.*ones(length(weights),1);
end

% C implementation
stat = mrLinearStat2d_mex(u, cube.st-1, cube.ed-1, weights(:), scalePenalties(:));

