/* H. Li - September 30, 2015 */

/* ---------- multiresolution linear statistics */
double mrLinearStat2d(const double *u, int m, int n,                 /* data */
                      const double *st, const double *ed, int nCube, /* system of cubes */
                      const double *weights,                         /* weights */
                      const double *scalePenalties)                  /* scale penalties */
{
    /* cumulative sum */
    double *cs = (double *)mxMalloc((m+1)*(n+1)*sizeof(double));
    for (int i = 0; i <= m; ++i) {
        cs[i] = 0.;
    }
    for (int j = 1; j <= n; ++j) {
        cs[j*(m+1)] = 0.;
    }
    for (int j = 0; j < n; ++j) {
        for (int i = 0; i < m; ++i) {
            cs[(i+1)+(j+1)*(m+1)] = u[i+j*m] + (cs[i+(j+1)*(m+1)] - cs[i+j*(m+1)]) + cs[(i+1)+j*(m+1)];
        }
    }
    
    /* compute statistic */
    double stat = -INFINITY;
    double sumCube; /* sum on each cube */
    for (int k = 0; k < nCube; ++k) {
        sumCube =   cs[((int)ed[k]+1)+((int)ed[k+nCube]+1)*(m+1)]
                  - cs[((int)st[k])  +((int)ed[k+nCube]+1)*(m+1)]
                  - cs[((int)ed[k]+1)+((int)st[k+nCube])*(m+1)]
                  + cs[((int)st[k])  +((int)st[k+nCube])*(m+1)];
        stat    = fmax(stat, fabs(sumCube)/weights[k] - scalePenalties[k]);
    }
    
    mxFree(cs);
    
    return stat;
}