function [ u ] = sobolevRegression1d(v, lambda, k)
%SOBLEVREG1D_SP Sobolev regularization in 1-d over splines using conjugate
%gradient methods (a.k.a. smoothing splines)
%   minimize 1/2 ||u - v||^2_2 + lambda/2 ||D^k u||^2_{L^2}
%   over u: [0, 1) --> R, periodic

% Housen Li @ MPIbpc
% 10.02.2015 create the code
% 05.01.2016 change name "sobolevReg1d_sp" --> "sobolevRegression1d"

sz = size(v);
v  = v(:); 
n  = numel(v);
fv = fft(v);

% B-spline bases matrix
x = linspace(0, 1, n+1);
x = x(1:end-1);
a = nbspline(x(:), 2*k, 0, n);
a = a(:);
% Q = sparse(toeplitz(a, a([1, end:-1:2])));
fQ  = fft(a);
fQt = fft(a([1, end:-1:2]));

% finite difference matrix
% D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
% D(1,n) = -1;
% D      = n * D;
fD  = fft([n; -n; zeros(n-2,1)]).^k;
fDt = fft([n; zeros(n-2,1); -n]).^k;

% Gramian matrix
a = zeros(1, n);
f0 = @(par) nbspline(par, k, 0, n);
for i = -(k-1):(k-1)
    fi = @(par) nbspline(par, k, i, n);
    ip = @(par) fi(par) .* f0(par);
    a(mod(i, n)+1) = quadgk(ip, 0, 1);
end
% G = sparse(toeplitz(a));
fG = fft(a(:));

% solve A c = (Q' Q + lambda D' G D) c = Q' v
fA                = fQt .* fQ + lambda * fDt .* fG .* fD;
fA(abs(fA) < eps) = 1;

u = real(ifft(fQ .* fQt .* fv ./ fA));
u = u - mean(u);
u = reshape(u, sz);

end

