function [ u ] = sobolevRegression2d(v, lambda, k, verbose)
%SOBOLEVREGRESSION2d L2-norm as data misfit penalty with homogeneous 
%Sobolev norm as regularization, i.e.
% 
%   minimize 1/2 ||u - v||^2 + lambda/2 ||D^k u||^2
%
%Usage:
% u = sobolevRegression2d(v, lambda, k, verbose);
%
%Input:
% v:        data
% lambda:   regularization parameter
% k:        Sobolev index
% verbose:  level of screen print
%
%Output:
% u:      solution (defined on [0, 1) and discretized on uniform grids)


% Housen Li @ MPIbpc
% 06.10.2015 create the code
% 07.01.2016 change name "sobolevRegression" --> "sobolevRegression2d"

% default input
if nargin < 4
    verbose = 0;
end

% initialization
[m, n] = size(v);
wr     = repmat(2*pi*circshift((-floor(m/2):1:(ceil(m/2)-1))', mod(m,2)), [1,n]);
wc     = repmat(2*pi*circshift(-floor(n/2):1:(ceil(n/2)-1), [0,mod(n,2)]), [m,1]);
w      = fftshift((wr.^2 + wc.^2).^k);

if verbose > 0
    fprintf('Sobolev inversion ...\n'); tic;
end

lambda = lambda/(m*n);
u = real(ifft2(fft2(v)./(1+lambda.*w)));

if verbose > 0
    fprintf('elapse %g\n', toc);
end


