function [ u ] = tikhonovInv( v, K, Kh, lambda, u_reg, u_init, maxit, tol, verbose )
%TIKHONOVINV Inversion by Tikhonov regularization
%   min_u || K u - v ||_2^2 + lambda || u - u_reg ||_2^2
%
%Algorithm: conjugate gradient (CG)
%   (I + 1/lambda K' K) u = 1/lambda K' v + u_reg 

% Housen Li @ MPIbpc
% 12.10.2015 built from M. Grasmair's code

% right hand side of equation
g = Kh(v)/lambda + u_reg;
% initialisation of CG
u = u_init;
forward_evaluation = K(u);
residual = g-u-Kh(forward_evaluation)/lambda;

descent_direction = residual;
nit = 0;
norm_residual = sum(sum(residual.^2));
if (norm_residual <= tol)
    not_converged = 0;
else
    not_converged = 1;
end
while (not_converged)
    forward_evaluation_d = K(descent_direction);
    R_descent_direction = descent_direction + Kh(forward_evaluation_d)/lambda;
    
    sigma = norm_residual/sum(sum(descent_direction.*R_descent_direction));
    u     = u + sigma*descent_direction;
    residual          = residual - sigma*R_descent_direction;
    norm_residual_new = sum(sum(residual.^2));
    beta              = norm_residual_new/norm_residual;
    descent_direction = residual + beta*descent_direction;
    norm_residual = norm_residual_new;
    if (nit >= maxit || norm_residual <= tol)
        not_converged = 0;
    end
    nit = nit + 1;
end

if (verbose)
    fprintf('      # CGs: %d\n', nit);
end
