function [ u ] = tvInversion1dADMM( f, Km, alpha )
%TVINVERSION1DADMM Summary of this function goes here
%   minimize 1/2 || Km u - f ||^2 + mu || D u ||_1

% H. Li @ MPIbpc
% 26.12.2015 build the code from M. Grasmair's code for 2 dimension

% default parameters
verbose = 0; % give some output during the computation
check   = 10;

%% initialisations and memory allocations
% convergence flag and step number for the main iteration
not_converged = 1;
iterate_main  = 0;
% store the (unfiltered) backprojection of the data for future use
f_back = Km'*f;
% initialise the solution u by (unfiltered) backprojection
u = f_back;
% initialise the auxiliary variable v as equal to u
v = u;
% initialisation of the Lagrange parameter
lambda = zeros(size(u));


%%%%%% TO BE DONE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% main iteration
while (not_converged)
    % store the old values of u and v
    u_old = u;
    v_old = v;
    
    current_verbose = verbose && (mod(iterate_main, check) == 0);
    if (current_verbose) 
        fprintf('  iteration %d ...\n', iterate_main);
    end
    
    %%% solve the denoising problem (update of u)
    % algorithm from "Bect, J., Blanc-F?raud, L., Aubert, G., and
    %   Chambolle, A., A $l$1-unified variational framework for image 
    %   restoration. Computer Vision - ECCV 2004 Volume 3024 of the series 
    %   Lecture Notes in Computer Science pp 1-13"
    % initialisations - in the case of large regularisation parameter, it
    % might make sense to improve the initialisation, else the number of
    % iterations will be huge
    u = denoiseTV(v+lambda, alpha/sigma, u, max_steps_denoising, eps_denoising, current_verbose);
    
    %%% invert the Radon transform (update of v)
    v = tikhonovInv(f, K, Kh, sigma, u-lambda, v, max_steps_CG, eps_CG, current_verbose);
    
    %%% update of lambda
    lambda = lambda + (v - u);
    
    iterate_main = iterate_main + 1;
    
    if (iterate_main >= max_steps)
        not_converged = 0;
    end
    increment_u    = max(max(abs(u_old-u)));
    increment_v    = max(max(abs(v_old-v)));
    difference_u_v = max(max(abs(u-v)));
    if (current_verbose)
        fprintf('      increment_u = %g, increment_v = %g, diff_u_v = %g\n',...
            increment_u, increment_v, difference_u_v);
    end
    
    if (increment_u <= eps_main && increment_v <= eps_main && difference_u_v <= eps_main)
        not_converged = 0;
    end
    
    if (vis_level > 1)
        figure(1);clf;
        subplot(131); imagesc(u);      axis off; colorbar; drawnow;
        subplot(132); imagesc(v);      axis off; colorbar; drawnow;
        subplot(133); imagesc(lambda); axis off; colorbar; drawnow;
    end
end

if (verbose)
    fprintf('number of main iterations: %d\n', iterate_main);
end

if (vis_level > 0)
    figure(1);clf;
    subplot(131); imagesc(u);      axis off; colorbar; drawnow;
    subplot(132); imagesc(v);      axis off; colorbar; drawnow;
    subplot(133); imagesc(lambda); axis off; colorbar; drawnow;
end
