function [ ndf ] = l2norm_der_sp(u, k, m)
%L2NORM_DER_SP Compute L2-norm of k-th derivative of splines of order m
%   ndf = l2norm_der_fd(u, k, m)
%   ndf = l2norm_der_fd(u, k)

% Housen Li @ MPIbpc 
% 18.01.2015 creat the code

eps_tol = 1e-12;

if nargin < 3
    m = 2*k;
end

u  = u(:); 
n  = numel(u);
fu = fft(u);

% finite differences
% D      = spdiags([ones(n,1),-ones(n,1)],[0,-1],n,n);
% D(1,n) = -1;
% D      = n*D;
fD  = fft([n; -n; zeros(n-2,1)]).^k;
% fDt = fft([n; zeros(n-2,1); -n]).^k;
fDt = conj(fD);

% B-spline bases matrix
x  = linspace(0, 1, n+1);
x  = x(1:end-1);
a  = nbspline(x(:), m, 0, n);
a  = a(:);
% Q = sparse(toeplitz(a, a([1, end:-1:2])));
fQ = fft(a);

% Gramian matrix
a = zeros(1, n);
l = m - k;
f0 = @(par) nbspline(par, l, 0, n);
for i = -(l-1):(l-1)
    fi = @(par) nbspline(par, l, i, n);
    ip = @(par) fi(par) .* f0(par);
    a(mod(i, n)+1) = quadgk(ip, 0, 1);
end
% G = sparse(toeplitz(a));
fG = fft(a(:));

fQaux                       = fQ;
fQaux(abs(fQaux) < eps_tol) = 1;

fc = fu ./ fQaux;

ndf = sqrt(real(fc' * (fDt .* fG .* fD .* fc))/n);

end

