function [ intval ] = pPartitionIntval( n, p )
%PPARTITIONINTVAL Generate p-partition interval system
%   intval = pPartitionIntval(n, p)

% Housen Li @ MPIbpc
% 02.06.2015 create the code

if nargin < 2
    p = 2; % default is dyadic 
end

maxTheta = ceil(log(n) / log(p));
ni       = (p^(maxTheta+1) - 1) / (p - 1);

intval.li = zeros(ni,1);
intval.ri = zeros(ni,1);

cnt = 1;
for theta = 0:maxTheta
    [lnd, rnd] = partitionFixLen(n, p^theta);
    nIntval    = length(lnd);
    
    intval.li(cnt:(cnt+nIntval-1)) = lnd;
    intval.ri(cnt:(cnt+nIntval-1)) = rnd;
 
    cnt = cnt + nIntval;
end

intval.li = intval.li(1:(cnt-1));
intval.ri = intval.ri(1:(cnt-1));

% remove redundancy
[intval.li, ind] = sort(intval.li);
intval.ri        = intval.ri(ind);
ind = (diff([intval.ri; 0]) ~= 0);
intval.li = intval.li(ind);
intval.ri = intval.ri(ind);
