function [ x ] = proj_ellipsoidvar(a, w, alpha, verbose )
%PROJ_ELLIPSOIDVAR Solve the projection on an ellipsoid by 
%   argmin      || x - a ||_2 
%   subject to  sum(x .* w .* x) <= alpha
%
%where w is a vector

% Housen Li @ MPIbpc 
% 18.01.2015 create the code

if nargin < 4
    verbose = 0;
end

% default parameters
maxit = 100;
tol   = 1e-8;

% initialization
lmb = 0;
fv  = sum(w.*a.^2) - alpha;  % function value
cnt = 0;

% iteration
while fv > 0
    dfv = 2*sum(w.^2.*a.^2./(1+lmb*w).^3); % derivative function value
    lmb = lmb + fv / dfv;
    fv  = sum(w.*(a./(1+lmb*w)).^2) - alpha;
    cnt = cnt +1;
    % stopping criteria
    if cnt > maxit || fv < tol
        break;
    end
end

x = a ./ (1 + lmb*w);

end