function [ uo ] = prox_inf_norm(ui, lambda)
%PROX_INF_NORM Proximal mapping of infinity norm
%   uo = prox_inf_norm(ui, lambda)
%
%   uo = argmin 1/(2 lambda) || u - ui ||_2^2 + || u ||_{infty}

% Housen Li @ MPIbpc 
% 20.10.2014 create the code


n  = numel(ui);
us = [sort(abs(ui(:)), 'descend'); 0];
cs = cumsum(us);

i0 = 0;
for i = 1:n
    if cs(i) - i*us(i+1) >= lambda
        i0 = i;
        break;
    end
end
if i0 == 0
    uo = 0 .* ui;
else
    th = (cs(i0) - lambda)/i0;
    uo = ui ./ max(abs(ui)/th, 1);
end

end

