function [ f ] = h_half_plus_s_minus_eps(x, s, w, k)
%H_HALF_PLUS_S_MINUS_EPS Function in H^{1/2+s-\epsilon}_{\diamond}(\T) for 
%\epsilon > 0
%   f = h_half_plus_s_minus_eps(x, s, w, k)
%
% Note 's' is a natural number
%
%Reference: 
%-Kuemmerer, L., Potts, D., Volkmer, T. Approximation of multivariate 
%  functions by trigonometric polynomials based on rank-1 lattice sampling. 
%  DFG-Schwerpunktprogramm 1324, Preprint 145, 2013
%-M. Griebel and J. Hamaekers. Fast discrete Fourier transform on 
%  generalized sparse grids. Univ. Bonn, INS Preprint No. 1305, 2013.

% Housen Li @ MPIbpc
% 08.03.2014 create the code

if nargin < 3
    w = 1;
end
if nargin < 4
    k = 0;
end

% periodic extension
x = mod(x, 1);

if k == 0
    % mean is zero
    % f = sign(x-1/2).*sin(2*pi.*x).^s + (1-(-1)^s)*2^(s-1)/pi*beta((s+1)/2, (s+1)/2);
    f = sign(x-1/2).*sin(2*pi*w.*x).^s + (1-(-1)^s)*2^(s-1)/pi*beta((s+1)/2, (s+1)/2)/w;
else
    % end with 0 indicates symbolic variables or functions
    syms x0 s0 w0
    f0  = sin(w0*x0)^s0;
    df0 = diff(f0, x0, k);
    df0 = subs(df0, x0, x);
    df0 = subs(df0, w0, 2*pi*w);
    df0 = double(subs(df0, s0, s));
    f   = df0.*sign(x-1/2);
end

end
